/*
 * Copyright 2008 Pentaho Corporation.  All rights reserved. 
 * This software was developed by Pentaho Corporation and is provided under the terms 
 * of the Mozilla Public License, Version 1.1, or any later version. You may not use 
 * this file except in compliance with the license. If you need a copy of the license, 
 * please go to http://www.mozilla.org/MPL/MPL-1.1.txt. The Original Code is the Pentaho 
 * BI Platform.  The Initial Developer is Pentaho Corporation.
 *
 * Software distributed under the Mozilla Public License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or  implied. Please refer to 
 * the license for the specific language governing your rights and limitations.
 */
package org.pentaho.iphone;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

/**
 * The purpose of this Filter is to redirect various requests to iphone JSPs
 *
 * @author Will Gorman (wgorman@pentaho.com)
 */
public class PentahoiPhoneFilter implements Filter {

  public void init(FilterConfig filterConfig) throws ServletException {
  }

  public void doFilter(ServletRequest req, ServletResponse res, FilterChain filterChain) throws IOException,
      ServletException {
    if (req instanceof HttpServletRequest) {
    	
      HttpServletRequest request = (HttpServletRequest)req;
      
      // check the user agent of the request
      
      if (request.getHeader("User-Agent") != null && request.getHeader("User-Agent").toLowerCase().indexOf("iphone") >= 0) {  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        if (request.getSession(true).getAttribute("iphone-view") == null) { //$NON-NLS-1$
          request.getSession(true).setAttribute("iphone-view", "true"); //$NON-NLS-1$ //$NON-NLS-2$
        }
      }
      
      // allow overriding of the iphone-view
      
      if (request.getParameter("iphone-view") != null) { //$NON-NLS-1$
        request.getSession(true).setAttribute("iphone-view", request.getParameter("iphone-view")); //$NON-NLS-1$ //$NON-NLS-2$
      }
      
      String servletPath = request.getServletPath();
      if ("true".equals(request.getSession(true).getAttribute("iphone-view"))) { //$NON-NLS-1$ //$NON-NLS-2$

    	// configure param pages
    	  
        request.setAttribute("default-parameter-xsl", "IPhoneDefaultParameterForm.xsl"); //$NON-NLS-1$ //$NON-NLS-2$
        
        // forward Login, Navigate, About, and Home to the iphone directory JSPs
        
        if (servletPath != null && 
            ( 
            servletPath.startsWith("/Login") || //$NON-NLS-1$
            servletPath.startsWith("/Navigate") || //$NON-NLS-1$
            servletPath.startsWith("/About") || //$NON-NLS-1$
            servletPath.startsWith("/Home") //$NON-NLS-1$
            )) {
          RequestDispatcher dispatcher = req.getRequestDispatcher("/iphone" + servletPath + ".jsp"); //$NON-NLS-1$ //$NON-NLS-2$
          dispatcher.forward(req, res);
          return;
        }
      }
    }
    filterChain.doFilter(req, res);
  }

  public void destroy() {
  }

}
